#include <ioavr.h>
#include "compiler.h"
#include "eeprom.h"

void Write_EEPROM(unsigned int Address, unsigned char Data)
{
	unsigned char istate;

    // Wait if write is under progress
    while ( EECR & (1 << EEPE) )
        {}; 

    EEAR =  Address;

    // Check if data is changed before writing
    // Enable EEPROM read.
    EECR |= (1 << EERE); 
    
    if(EEDR==Data) // return without writing if data is unchanged
        return;  

    EEDR = Data;

    // Important to disable interrupts during writing to prevent corruption
	istate = __save_interrupt();
	__disable_interrupt();


    // Enable EEPROM write
    EECR = (1 << EEMPE); 
    // Start EEPROM write
    EECR |= (1 << EEPE); 

    // Set address register to zero just in case
    EEAR = 0;     

	__restore_interrupt(istate);
}

unsigned char Read_EEPROM(unsigned int Address)
{
    unsigned char Data;
    
    // Wait if write is under progress
    while ( EECR & (1 << EEPE) )
        {}; 

    EEAR = Address;

    // Enable EEPROM read.
    EECR |= (1 << EERE); 

    Data = EEDR;
    
    return (Data);
}

