#include <ioavr.h>
#include "compiler.h"
#include "conf_usb.h"

#define PORT_RESET PORTD_Bit1
#define DDR_RESET DDRD_Bit1

U8 isp_enter_progmode();
#define isp_leave_progmode() PORT_RESET = 1
U8 isp_verify_page(U16 addr, U8 code * buf, U8 len);
U8 isp_write_page(U16 addr, U8 code * buf, U8 len);
U8 isp_command(U8 b1, U8 b2, U8 b3, U8 b4);

#define isp_chip_erase()       isp_command(0xAC,0x80,0,0)
#define isp_load_ext_addr(a)   isp_command(0x4D,0,a,0)
#define isp_wait_rdy()         do {isp_command(0xF0,0,0,0);} while(SPDR&1)
#define isp_write_fuse_0(v)    isp_command(0xAC,0xA0,0,v)
#define isp_write_fuse_1(v)    isp_command(0xAC,0xA8,0,v)
#define isp_write_fuse_2(v)    isp_command(0xAC,0xA4,0,v)
#define isp_read_fuse_0()      isp_command(0x50,0x00,0,0)
#define isp_read_fuse_1()      isp_command(0x58,0x08,0,0)
#define isp_read_fuse_2()      isp_command(0x50,0x08,0,0)
#define isp_read_sig(v)       isp_command(0x30,0x00,v,0)


U8 isp_ctrl()
{
    volatile U8 sel=0;
    volatile U8 res;
    U8 v=0;
    U32 y,x=0;
    
    asm("nop");
    isp_enter_progmode();
    
    switch(sel)
    {
        case 1: res=isp_read_fuse_0(); break;
        case 2: res=isp_read_fuse_1(); break;
        case 3: res=isp_read_fuse_2(); break;
        case 4: res=isp_write_fuse_0(v);break;
        case 5: res=isp_write_fuse_1(v);break;
        case 6: res=isp_write_fuse_2(v);break;
        case 7:  res=isp_read_sig(2);
            x=SPDR;
            res=isp_read_sig(1);
            y=SPDR;x|=y<<8;
            res=isp_read_sig(0);
            y=SPDR;x|=y<<16;
            res=y==0x001e9482;
            break;
    default: break;
    }
    x=SPDR;
    asm("nop");
    sel=res;                
    res=v;
    v=x;
    return 0;
}

/**
Program the the target connected to ISP lines
with len bytes of pbuf code.
*/
U8 isp_prog(U8 code * pbuf, U16 len,U16 ofs)
{
    U8 a;
    U8 isp_pagesize;
    
    if (len&1) return 0xff; // len must be pair.
    
    /* Enable pins and SPI Master, set clock rate fck/16 */
    DDR_SPI = (1<<DD_MOSI)|(1<<DD_SCK)|(1<<DD_SS);
    SPCR = (1<<SPE)|(1<<MSTR)|(1<<SPIE)|(1<<SPR0);//

    if (isp_enter_progmode())
    {
        return 1;
    }
    
    isp_pagesize=64;
    
    while(len>0)
    {
        a=isp_write_page(ofs,pbuf,isp_pagesize);
        if(a)
            return a;
        a=isp_verify_page(ofs,pbuf,isp_pagesize);
        if(a)
            return a;
        len -=isp_pagesize*2;
        ofs+=isp_pagesize*2;
        if (len<isp_pagesize*2)
            isp_pagesize=len*2;
    }
    isp_leave_progmode();
    return 0;
}

U8 isp_command(U8 b1, U8 b2, U8 b3, U8 b4)
{
    SPDR = b1;   while(!(SPSR & (1<<SPIF)));
    SPDR = b2;   while(!(SPSR & (1<<SPIF)));
    SPDR = b3;   while(!(SPSR & (1<<SPIF)));
    b3 = SPDR;
    SPDR = b4;   while(!(SPSR & (1<<SPIF)));
    return b3 != b2;
}


U8 isp_write_page(U16 addr, U8 code * buf, U8 len)
{
    U8 w;
    U8 ofs=0;
    
    for(ofs=0;ofs<len;ofs++)
    {
        w=*(buf++);
        isp_wait_rdy();
        if (isp_command(0x40,0,ofs, w))
            return 1;
        w=*(buf++);
        isp_wait_rdy();
        if (isp_command(0x48,0,ofs, w))
            return 1;
    }    
    isp_wait_rdy();
    if (isp_command(0x4C, (addr>8) & 0xff, addr & 0xff,0)) // latch page
        return 1;
    return 0;
}

U8 isp_verify_page(U16 addr, U8 code * buf, U8 len)
{
    U8 w;
    
    while(len-->0)
    {
        w=*(buf++);
        isp_wait_rdy();
        if (isp_command(0x20,(addr>>8)&0xff,addr&0xff, 0))
            return 1;
        if (SPDR!=w) return 2;
        w=*(buf++);
        isp_wait_rdy();
        if (isp_command(0x28,(addr>>8)&0xff,addr&0xff, 0))
            return 1;
        if (SPDR!=w) return 2;
        addr++;
    }
    
    return 0;
}

void sleep(U8 ms)
{
    while(ms--)
        __delay_cycles(8000);
}

U8 isp_enter_progmode()
{
            /* Set MOSI and SCK output, all others input */
        DDR_SPI = (1<<DD_MOSI)|(1<<DD_SCK)|(1<<DD_SS);

        /* Enable SPI, Master, set clock rate fck/16 */
        SPCR = (1<<SPE)|(1<<MSTR)|(1<<SPR0);//

    DDR_RESET =1;
    
    do
    {
        PORT_RESET = 1;
        __delay_cycles(10);
        PORT_RESET = 0;
   
        sleep(25); // wait atleast 20 ms
        //TODO: implement timeout
    }
    while (isp_command(0xAC, 0x53,0,0));
    
    return 0;
}
