#include <ioavr.h>

#define RPC_BOOTLOADER()  (rpc_send(1,0,0))
#define RPC_SET_GUITAR(x) (rpc_send(2,1,x))
#define RPC_SET_REPORT(x) (rpc_send(3,1,x))

void rpc_init(void)
{
#ifdef RPC_MASTER
    /* Set MOSI and SCK output, all others input */
    DDR_SPI = (1<<DD_MOSI)|(1<<DD_SCK);
    /* Enable SPI, Master, set clock rate fck/16 */
    SPCR = (1<<SPE)|(1<<MSTR)|(1<<SPR0);
#else
    /* Set MISO output, all others input */
    DDR_SPI = (1<<DD_MISO);
    /* Enable SPI */
    SPCR = (1<<SPE);
#endif
}

char rpc_onebuf;
char * rpc_tx_data=0;
char * rpc_rx_data=0;
char * rpc_rx_count=0;
char rpc_remains=0;

void rpc_prepare_read(char * rxbuf, char len)
{
    while (rpc_remains);  // wait for queue to free up;

    rpc_rx_data=rxbuf;
    rpc_rx_count=len;
}

void rpc_send(char msg, char len, char * data)
{
    while (rpc_remains);  // wait for queue to free up;
    
    set_spi_ss();
    SPDR = msg;
    if (len==1) 
    {
        rpc_onebuf=data;
        rpc_tx_data=&rpc_onebuf;
    }
    else
        rpc_tx_data = data;
    rpc_remains=len;
}



#ifdef AVRGCC
 ISR(SPI_STC_vect)
#else
#pragma vector = SPI_STC_vect
__interrupt void spi_interrupt()
#endif
{
    if(rpc_rx_count>0)
    {
        *rpc_rx_data=SPDR;
        rpc_rx_data++;
        rpc_rx_count--;
    }
       
    if (rpc_remains==0)
    {
        clear_spi_ss();
    }
    else
    {
        SPDR = *rpc_tx_data;
        rpc_tx_data++;
        rpc_remains--;
    }
}